"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const fs = require("fs");
const path = require("path");
const SymbolTree = require("symbol-tree");
const base_1 = require("../base");
const fetchChildren_1 = require("../utils/data/fetchChildren");
const fetchCompleted_1 = require("../utils/data/fetchCompleted");
const fetchInitiatives_1 = require("../utils/data/fetchInitiatives");
const getEmptyCalendarObject_1 = require("../utils/roadmap/getEmptyCalendarObject");
const prepareInitiativesData_1 = require("../utils/roadmap/prepareInitiativesData");
const teamClosedByWeek_1 = require("../utils/roadmap/teamClosedByWeek");
class Roadmap extends base_1.default {
    constructor() {
        super(...arguments);
        this.exportData = (issuesTree, node) => {
            const jsonObject = [];
            for (const initiative of issuesTree.childrenIterator(node)) {
                const epics = [];
                for (const epic of issuesTree.childrenIterator(initiative)) {
                    const stories = [];
                    for (const story of issuesTree.childrenIterator(epic)) {
                        stories.push(story);
                    }
                    epic.children = stories;
                    epics.push(epic);
                }
                initiative.children = epics;
                jsonObject.push(initiative);
            }
            return jsonObject;
        };
        this.showArtifactsTable = (roadmapArtifact, type) => {
            const columnsByTeam = {
                name: {
                    header: "Team",
                    minWidth: "10",
                    get: (row) => {
                        if (row.name === null) {
                            return "TOTAL";
                        }
                        return row.name;
                    }
                }
            };
            for (let week of roadmapArtifact.byTeam[0].weeks) {
                const weekId = week.weekStart.slice(0, 10);
                columnsByTeam[weekId] = { header: weekId };
            }
            cli_ux_1.default.table(roadmapArtifact.byTeam.map((team) => {
                const teamData = Object.assign({}, team);
                for (let week of team.weeks) {
                    const weekId = week.weekStart.slice(0, 10);
                    teamData[weekId] = week[type].count;
                }
                return teamData;
            }), columnsByTeam);
            const columnsByInitiative = {
                prefix: {
                    header: "-",
                    get: (row) => {
                        switch (row.level) {
                            case 1:
                                return "-----";
                            case 2:
                                return " |---";
                            case 3:
                                return " | |-";
                        }
                    }
                },
                type: {
                    header: "Type",
                    minWidth: "10",
                    get: (row) => {
                        return row.fields.issuetype.name;
                    }
                },
                key: {
                    header: "Key"
                },
                title: {
                    header: "Title",
                    get: (row) => {
                        return row.fields.summary;
                    }
                },
                state: {
                    header: "State",
                    minWidth: "10",
                    get: (row) => {
                        return row.fields.status.statusCategory.name;
                    }
                },
                pts: {
                    header: "Pts",
                    get: (row) => {
                        if (row.isLeaf) {
                            if (row.metrics.missingPoints) {
                                return "-";
                            }
                            return row.metrics[type].total;
                        }
                        return "";
                    }
                },
                progress: {
                    header: "Progress",
                    minWidth: "5",
                    get: (row) => {
                        if (!row.isLeaf) {
                            let progress = "0%";
                            let missing = "";
                            if (row.metrics[type].missing > 0) {
                                missing =
                                    " (" +
                                        row.metrics[type].missing +
                                        " open issues without estimate)";
                            }
                            if (row.metrics[type].total > 0) {
                                progress =
                                    Math.round(((row.metrics[type].completed * 100) /
                                        row.metrics[type].total) *
                                        100) /
                                        100 +
                                        "%";
                            }
                            return (row.metrics[type].completed +
                                "/" +
                                row.metrics[type].total +
                                " - " +
                                progress +
                                missing);
                        }
                        return "";
                    }
                }
            };
            for (let week of roadmapArtifact.byInitiative[0].weeks) {
                const weekId = week.weekStart.slice(0, 10);
                columnsByInitiative[weekId] = { header: weekId };
            }
            cli_ux_1.default.table(roadmapArtifact.byInitiative.map((initiative) => {
                const initiativeData = Object.assign({}, initiative);
                for (let week of initiative.weeks) {
                    const weekId = week.weekStart.slice(0, 10);
                    initiativeData[weekId] = week[type].count;
                }
                return initiativeData;
            }), columnsByInitiative);
        };
    }
    async run() {
        const { flags } = this.parse(Roadmap);
        let { type, cache } = flags;
        const userConfig = this.userConfig;
        const cacheDir = this.config.configDir + "/cache/";
        if (cache) {
            this.log("==================================================================================");
            this.log("Will be fetching data from cache. NO CALLS WILL BE MADE TO JIRA TO REFRESH DATA ");
            this.log("==================================================================================");
        }
        // Creates an array of all closed issues across all teams
        let closedIssues = [];
        for (let team of userConfig.teams) {
            const teamIssues = await fetchCompleted_1.default(userConfig, this.config.configDir + "/cache/", team.name);
            closedIssues = [...closedIssues, ...teamIssues];
        }
        const emptyCalendar = getEmptyCalendarObject_1.default(closedIssues, userConfig);
        const closedIssuesByWeekAndTeam = teamClosedByWeek_1.default(closedIssues, userConfig, emptyCalendar);
        const initiativesIssues = await fetchInitiatives_1.default(userConfig, cacheDir, cache);
        // Structure the issues in an actual tree object for easier traversing
        //Note: Parent field (if parent is EPIC): customfield_10314
        //Note: Parent field (if parent is INITIATIVE): customfield_11112
        const issuesTree = new SymbolTree();
        const treeRoot = {};
        for (let initiative of initiativesIssues) {
            issuesTree.appendChild(treeRoot, initiative);
            const children = await fetchChildren_1.default(userConfig, initiative.key, cacheDir, cache);
            for (let l1child of children.filter((ic) => ic.fields[userConfig.jira.fields.parentInitiative] === initiative.key)) {
                issuesTree.appendChild(initiative, l1child);
                for (let l2child of children.filter((ic) => ic.fields[userConfig.jira.fields.parentEpic] === l1child.key)) {
                    issuesTree.appendChild(l1child, l2child);
                }
            }
        }
        // Update all of the tree nodes with actual metrics
        prepareInitiativesData_1.default(issuesTree, treeRoot, 0, closedIssues, emptyCalendar, userConfig);
        const closedIssuesByWeekAndInitiative = this.exportData(issuesTree, treeRoot);
        const roadmapArtifact = {
            byTeam: closedIssuesByWeekAndTeam,
            byInitiative: closedIssuesByWeekAndInitiative
        };
        this.showArtifactsTable(roadmapArtifact, type);
        const issueFileStream = fs.createWriteStream(path.join(cacheDir, "roadmap-artifacts.json"), { flags: "w" });
        issueFileStream.write(JSON.stringify(roadmapArtifact));
        issueFileStream.end();
    }
}
exports.default = Roadmap;
Roadmap.description = "Builds a roadmap from a set of issues";
Roadmap.flags = Object.assign(Object.assign({}, base_1.default.flags), { help: command_1.flags.help({ char: "h" }), type: command_1.flags.string({
        char: "t",
        description: "Use issues of points for metrics",
        options: ["issues", "points"],
        default: "points"
    }), cache: command_1.flags.boolean({
        char: "c",
        description: "Use cached version of the child issues (mostly useful for dev)",
        default: false
    }) });
